@@ -15,22 +15,16 @@
  *
  * The Initial Developer of the Original Code is
  * Wladimir Palant.
- * Portions created by the Initial Developer are Copyright (C) 2006-2007
+ * Portions created by the Initial Developer are Copyright (C) 2006-2008
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *
  * ***** END LICENSE BLOCK ***** */
 
-var abp = Components.classes["@mozilla.org/adblockplus;1"].createInstance();
-while (abp && !("getString" in abp))
-  abp = abp.wrappedJSObject;    // Unwrap component
+var abp = Components.classes["@mozilla.org/adblockplus;1"].createInstance().wrappedJSObject;
 
 function fillInVersion() {
   var versionField = document.getElementById("version");
-  var version = abp.getInstalledVersion();
-  if (version)
-    versionField.setAttribute("value", versionField.getAttribute("value") + " " + version);
-  else
-    versionField.parentNode.removeChild(versionField);
+  versionField.value = abp.getInstalledVersion();
 }
