@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Wladimir Palant.
- * Portions created by the Initial Developer are Copyright (C) 2006-2007
+ * Portions created by the Initial Developer are Copyright (C) 2006-2008
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -24,9 +24,7 @@
 
 var abp = null;
 try {
-  abp = Components.classes["@mozilla.org/adblockplus;1"].createInstance();
-  while (abp && !("getString" in abp))
-    abp = abp.wrappedJSObject;    // Unwrap Adblock Plus component
+  abp = Components.classes["@mozilla.org/adblockplus;1"].createInstance().wrappedJSObject;
 
   if (!abp.prefs.initialized)
     abp = null;
@@ -61,7 +59,7 @@ function abpInit() {
   }
 
   // Install context menu handler
-  var contextMenu = document.getElementById("contentAreaContextMenu") || document.getElementById("messagePaneContext");
+  var contextMenu = document.getElementById("contentAreaContextMenu") || document.getElementById("messagePaneContext") || document.getElementById("popup_content");
   if (contextMenu) {
     contextMenu.addEventListener("popupshowing", abpCheckContext, false);
   
@@ -71,18 +69,18 @@ function abpInit() {
     contextMenu.appendChild(document.getElementById("abp-image-menuitem"));
   }
 
-  // Check whether Adblock is installed and uninstall
-  // Delay it so the browser window will be displayed before the warning
-  if (abp && !abpPrefs.checkedadblockinstalled)
-    setTimeout(abpCheckExtensionConflicts, 0);
+  // First run actions
+  if (abp && !("doneFirstRunActions" in abpPrefs) && abp.versionComparator.compare(abpPrefs.lastVersion, "0.0") <= 0)
+  {
+    // Don't repeat first run actions if new window is opened
+    abpPrefs.doneFirstRunActions = true;
 
-  // Install toolbar button in Firefox if necessary
-  if (abp && !abpPrefs.checkedtoolbar)
+    // Add ABP icon to toolbar if necessary
     setTimeout(abpInstallInToolbar, 0);
 
-  // Let user choose subscriptions on first start
-  if (abp && abpPrefs.showsubscriptions)
+    // Show subscriptions dialog if the user doesn't have any subscriptions yet
     setTimeout(abpShowSubscriptions, 0);
+  }
 
   // Move toolbar button to a correct location in Mozilla/SeaMonkey
   var button = document.getElementById("abp-toolbarbutton");
@@ -129,10 +127,8 @@ function abpGetBrowser() {
     return getBrowser();
   else if ("messageContent" in window)
     return window.messageContent;
-  else if (document.getElementById("frame_main_pane"))
-    return document.getElementById("frame_main_pane");
   else
-    return null;
+    return document.getElementById("frame_main_pane") || document.getElementById("browser_content");
 }
 
 function abpReloadPrefs() {
@@ -146,31 +142,32 @@ function abpReloadPrefs() {
 
     label = abp.getString("status_" + state + "_label");
 
-    if (state == "active") {
-      var location = null;
-      if ("currentHeaderData" in window && "content-base" in currentHeaderData) {
+    if (state == "active")
+    {
+      let location = null;
+      if ("currentHeaderData" in window && "content-base" in currentHeaderData)
+      {
         // Thunderbird blog entry
-    
-        location = abp.unwrapURL(currentHeaderData["content-base"].headerValue);
+        location = currentHeaderData["content-base"].headerValue;
       }
-      else if ("gDBView" in window) {
+      else if ("gDBView" in window)
+      {
         // Thunderbird mail/newsgroup entry
-
-        try {
+        try
+        {
           var msgHdr = gDBView.hdrForFirstSelectedMessage;
           var headerParser = Components.classes["@mozilla.org/messenger/headerparser;1"]
                                       .getService(Components.interfaces.nsIMsgHeaderParser);
           var emailAddress = headerParser.extractHeaderAddressMailboxes(null, msgHdr.author);
-          if (emailAddress) {
+          if (emailAddress)
             location = 'mailto:' + emailAddress.replace(/^[\s"]+/, "").replace(/[\s"]+$/, "").replace(' ', '%20');
-          }
         }
         catch(e) {}
       }
-      else {
+      else
+      {
         // Firefox web page
-
-        location = abp.unwrapURL(abpGetBrowser().contentWindow.location.href);
+        location = abpGetBrowser().contentWindow.location.href;
       }
 
       if (location && abp.policy.isWhitelisted(location))
@@ -198,8 +195,8 @@ function abpReloadPrefs() {
       else
         element.hidden = !abpPrefs.showintoolbar;
 
-      // HACKHACK: Show status bar icon in SeaMonkey Mail and Songbird instead of toolbar icon
-      if (element.hidden && (element.tagName == "statusbarpanel" || element.tagName == "vbox") && (document.getElementById("msgToolbar") || document.getElementById("frame_main_pane_html")))
+      // HACKHACK: Show status bar icon in SeaMonkey Mail and Prism instead of toolbar icon
+      if (element.hidden && (element.tagName == "statusbarpanel" || element.tagName == "vbox") && (document.getElementById("msgToolbar") || location.host == "webrunner"))
         element.hidden = !abpPrefs.showintoolbar;
 
       if (abpOldShowInToolbar != abpPrefs.showintoolbar)
@@ -227,7 +224,10 @@ function abpReloadPrefs() {
   updateElement(button);
   if (button) {
     if (button.hasAttribute("context") && abpPrefs.defaulttoolbaraction == 0)
+    {
       button.setAttribute("popup", button.getAttribute("context"));
+      button.removeAttribute("type");
+    }
     else
       button.removeAttribute("popup");
   }
@@ -309,55 +309,6 @@ function abpGetPaletteButton() {
   return null;
 }
 
-// Check whether Adblock is installed and uninstall
-function abpCheckExtensionConflicts() {
-  // Make sure not to run this twice
-  abpPrefs.checkedadblockinstalled = true;
-  abpPrefs.save();
-
-  if ("@mozilla.org/adblock;1" in Components.classes) {
-    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                                  .getService(Components.interfaces.nsIPromptService);
-    // Adblock is installed
-    if ("@mozilla.org/extensions/manager;1" in Components.classes) {
-      // Extension Manager available, ask whether to uninstall
-      var result = promptService.confirm(window, abp.getString("uninstall_adblock_title"),
-                                         abp.getString("uninstall_adblock_text"));
-      if (!result)
-        return;
-
-      try {
-        var id = Components.ID("{34274bf4-1d97-a289-e984-17e546307e4f}");
-        var extensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
-                                         .getService(Components.interfaces.nsIExtensionManager);
-
-        if ('uninstallItem' in extensionManager) {
-          // FF 1.1+
-          extensionManager.uninstallItem(id);
-        }
-        else {
-          // FF 1.0
-          // This seems to fail with the error "this._ds has no properties",
-          // but only if the check isn't done immediately after installation.
-          extensionManager.uninstallExtension(id);
-        }
-        promptService.alert(window, abp.getString("uninstall_adblock_title"),
-                                    abp.getString("uninstall_adblock_success"));
-      }
-      catch (e) {
-        dump("Adblock Plus: error uninstalling Adblock, " + e + "\n");
-        promptService.alert(window, abp.getString("uninstall_adblock_title"),
-                                    abp.getString("uninstall_adblock_error"));
-      }
-    }
-    else {
-      // No extension manager, recomend manual uninstall
-      promptService.alert(window, abp.getString("uninstall_adblock_title"),
-                                  abp.getString("uninstall_adblock_manually"));
-    }
-  }
-}
-
 // Check whether we installed the toolbar button already
 function abpInstallInToolbar() {
   if (!document.getElementById("abp-toolbarbutton")) {
@@ -407,26 +358,26 @@ function abpInstallInToolbar() {
       }
     }
   }
-
-  // Make sure not to run this twice
-  if (!abpPrefs.checkedtoolbar) {
-    abpPrefs.checkedtoolbar = true;
-    abpPrefs.save();
-  }
 }
 
 // Let user choose subscriptions on first start unless he has some already
-function abpShowSubscriptions() {
-  // Make sure not to run this twice
-  abpPrefs.showsubscriptions = false;
-  abpPrefs.save();
-
+function abpShowSubscriptions()
+{
   // Look for existing subscriptions
-  for (var i = 0; i < abpPrefs.subscriptions.length; i++)
-    if (!abpPrefs.subscriptions[i].special)
+  for each (let subscription in abp.filterStorage.subscriptions)
+    if (subscription instanceof abp.DownloadableSubscription)
       return;
 
-  window.openDialog("chrome://adblockplus/content/tip_subscriptions.xul", "_blank", "chrome,centerscreen,resizable=no,dialog=no");
+  let browser = abpGetBrowser();
+  if ("addTab" in browser)
+  {
+    // We have a tabbrowser
+    browser.selectedTab = browser.addTab("chrome://adblockplus/content/tip_subscriptions.xul");
+  }
+  else
+  {
+	window.openDialog("chrome://adblockplus/content/tip_subscriptions.xul", "_blank", "chrome,centerscreen,resizable=no,dialog=no");
+  }
 }
 
 function abpFillTooltip(ev) {
@@ -458,15 +409,15 @@ function abpFillTooltip(ev) {
       var locations = data.getAllLocations();
 
       var blocked = 0;
-      var filters = new abp.HashTable();
+      var filterCount = {__proto__: null};
       for (i = 0; i < locations.length; i++) {
-        if (locations[i].filter && locations[i].filter.type != "whitelist")
+        if (locations[i].filter && !(locations[i].filter instanceof abp.WhitelistFilter))
           blocked++;
         if (locations[i].filter) {
-          if (locations[i].filter.text in filters)
-            filters[locations[i].filter.text].value++;
+          if (locations[i].filter.text in filterCount)
+            filterCount[locations[i].filter.text]++;
           else
-            filters[locations[i].filter.text] = {value:1};
+            filterCount[locations[i].filter.text] = 1;
         }
       }
 
@@ -475,9 +426,9 @@ function abpFillTooltip(ev) {
       document.getElementById("abp-tooltip-blocked").setAttribute("value", blockedStr);
 
       var filterSort = function(a, b) {
-        return filters[b].value - filters[a].value;
+        return filterCount[b] - filterCount[a];
       };
-      for (var filter in filters)
+      for (var filter in filterCount)
         activeFilters.push(filter);
       activeFilters = activeFilters.sort(filterSort);
     }
@@ -491,7 +442,7 @@ function abpFillTooltip(ev) {
 
       for (var i = 0; i < activeFilters.length && i < 3; i++) {
         var descr = document.createElement("description");
-        descr.setAttribute("value", activeFilters[i] + " (" + filters[activeFilters[i]].value + ")");
+        descr.setAttribute("value", activeFilters[i] + " (" + filterCount[activeFilters[i]] + ")");
         filtersContainer.appendChild(descr);
       }
       if (activeFilters.length > 3) {
@@ -534,12 +485,10 @@ function abpFillPopup(popup) {
   var site = null;
   if ("currentHeaderData" in window && "content-base" in currentHeaderData) {
     // Thunderbird blog entry
-
     location = abp.unwrapURL(currentHeaderData["content-base"].headerValue);
   }
   else if ("gDBView" in window) {
     // Thunderbird mail/newsgroup entry
-
     try {
       var msgHdr = gDBView.hdrForFirstSelectedMessage;
       var headerParser = Components.classes["@mozilla.org/messenger/headerparser;1"]
@@ -554,30 +503,34 @@ function abpFillPopup(popup) {
 
     if (site) {
       whitelistItemSite.pattern = "@@|mailto:" + site.replace(' ', '%20') + "|";
-      whitelistItemSite.setAttribute("checked", abpHasPattern(whitelistItemSite.pattern));
+      whitelistItemSite.setAttribute("checked", abpHasFilter(whitelistItemSite.pattern));
       whitelistItemSite.setAttribute("label", whitelistItemSite.getAttribute("labeltempl").replace(/--/, site));
     }
   }
   else {
     // Firefox web page
-
     location = abp.unwrapURL(abpGetBrowser().contentWindow.location.href);
   }
 
   if (!site && location) {
     if (abp.policy.isBlockableScheme(location)) {
-      var url = location.replace(/\?.*/, '');
-      var host = abp.makeURL(location);
-      if (host)
-        host = host.host;
+      let ending = "|";
+      if (location instanceof Components.interfaces.nsIURL && location.query)
+      {
+        location.query = "";
+        ending = "?";
+      }
+
+      let url = location.spec;
+      let host = location.host;
       site = url.replace(/^([^\/]+\/\/[^\/]+\/).*/, "$1");
 
       whitelistItemSite.pattern = "@@|" + site;
-      whitelistItemSite.setAttribute("checked", abpHasPattern(whitelistItemSite.pattern));
+      whitelistItemSite.setAttribute("checked", abpHasFilter(whitelistItemSite.pattern));
       whitelistItemSite.setAttribute("label", whitelistItemSite.getAttribute("labeltempl").replace(/--/, host));
 
-      whitelistItemPage.pattern = "@@|" + url + "|";
-      whitelistItemPage.setAttribute("checked", abpHasPattern(whitelistItemPage.pattern));
+      whitelistItemPage.pattern = "@@|" + url + ending;
+      whitelistItemPage.setAttribute("checked", abpHasFilter(whitelistItemPage.pattern));
     }
     else
       location = null;
@@ -640,12 +593,22 @@ function abpToggleSidebar() {
     else
       abpDetachedSidebar = window.openDialog("chrome://adblockplus/content/sidebarDetached.xul", "_blank", "chrome,resizable,dependent,dialog=no,width=600,height=300");
   }
+
+  let menuItem = document.getElementById("abp-blockableitems");
+  if (menuItem)
+    menuItem.setAttribute("checked", abpIsSidebarOpen());
 }
 
-// Checks whether the specified pattern exists in the list
-function abpHasPattern(pattern) {
-  for (var i = 0; i < abpPrefs.userPatterns.length; i++)
-    if (abpPrefs.userPatterns[i].text == pattern)
+/**
+ * Checks whether the specified user-defined filter exists
+ *
+ * @param {String} filter   text representation of the filter
+ */
+function abpHasFilter(filter)
+{
+  filter = abp.Filter.fromText(filter);
+  for each (let subscription in abp.filterStorage.subscriptions)
+    if (subscription instanceof abp.SpecialSubscription && subscription.filters.indexOf(filter) >= 0)
       return true;
 
   return false;
@@ -676,7 +639,7 @@ function abpTogglePattern(text, insert) {
 
 // Handle clicks on the Adblock statusbar panel
 function abpClickHandler(e) {
-  if (e.button == 0 && new Date().getTime() - abpLastDrag > 100)
+  if (e.button == 0)
     abpExecuteAction(abpPrefs.defaultstatusbaraction);
   else if (e.button == 1)
     abpTogglePref("enabled");
@@ -694,76 +657,23 @@ function abpExecuteAction(action) {
   if (action == 1)
     abpToggleSidebar();
   else if (action == 2)
-    abpSettings();
+    abp.openSettingsDialog();
   else if (action == 3)
     abpTogglePref("enabled");
 }
 
-// Handles Drag&Drop of links and images to the Adblock statusbar panel
-function abpDragHandler(e) {
-  if (!abp)
-    return;
-
-  var dragService = Components.classes['@mozilla.org/widget/dragservice;1']
-                              .getService(Components.interfaces.nsIDragService);
-  var session = dragService.getCurrentSession();
-  if (!session)
-    return;
-
-  var link = null;
-  for (var node = new XPCNativeWrapper(session.sourceNode); node && !link; node = node.parentNode) {
-    if (node instanceof HTMLAnchorElement)
-      link = abp.unwrapURL(node.href);
-    else if (node instanceof HTMLImageElement)
-      link = abp.unwrapURL(node.src);
-  }
-
-  if (e.type == "dragover")
-    session.canDrop = (link != null);
-  else if (link)
-    abpSettings(link);
-
-  e.preventDefault();
-  e.stopPropagation();
-}
-
-var abpDraggingX = -1;
-var abpLastDrag = -1;
-
-// Allows activating/deactivating with a drag gesture on the Adblock status bar item
-function abpMouseHandler(e) {
-  if (!abp || e.button != 0)
-    return;
-
-  if (e.type == "mousedown") {
-    abpDraggingX = e.clientX;
-    e.target.addEventListener("mouseup", abpMouseHandler, false);
-    e.target.addEventListener("mouseout", abpMouseHandler, false);
-  }
-  else if (e.type == "mouseout" || e.type == "mouseup") {
-    e.target.removeEventListener("mouseup", abpMouseHandler, false);
-    e.target.removeEventListener("mouseout", abpMouseHandler, false);
-    if (e.type == "mouseup" && abpDraggingX >= 0 && Math.abs(e.clientX - abpDraggingX) > 10) {
-      abpPrefs.enabled = !abpPrefs.enabled;
-      abpPrefs.save();
-      abpLastDrag = new Date().getTime();
-    }
-    abpDraggingX = -1;
-  }
-}
-
 // Retrieves the image URL for the specified style property
 function abpImageStyle(computedStyle, property) {
   var value = computedStyle.getPropertyCSSValue(property);
   if (value.primitiveType == CSSPrimitiveValue.CSS_URI)
-    return abp.unwrapURL(value.getStringValue());
+    return abp.unwrapURL(value.getStringValue()).spec;
 
   return null;
 }
 
 // Hides the unnecessary context menu items on display
 function abpCheckContext() {
-  var contextMenu = document.getElementById("contentAreaContextMenu") || document.getElementById("messagePaneContext");
+  var contextMenu = document.getElementById("contentAreaContextMenu") || document.getElementById("messagePaneContext") || document.getElementById("popup_content");
   var target = document.popupNode;
 
   var nodeType = null;
@@ -828,14 +738,6 @@ function abpCheckContext() {
 
 // Bring up the settings dialog for the node the context menu was referring to
 function abpNode(data) {
-  if (data)
-    abpSettings(data.location);
-}
-
-// Open the settings window.
-function abpSettings(url) {
-  if (!abp)
-    return;
-
-  abp.openSettingsDialog(abpGetBrowser().contentWindow, url);
+  if (abp && data)
+    openDialog("chrome://adblockplus/content/composer.xul", "_blank", "chrome,centerscreen,resizable,dialog=no,dependent", abpGetBrowser().contentWindow, data);
 }
