@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Wladimir Palant.
- * Portions created by the Initial Developer are Copyright (C) 2006-2007
+ * Portions created by the Initial Developer are Copyright (C) 2006-2008
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -24,9 +24,7 @@
 
 var abp = null;
 try {
-  abp = Components.classes["@mozilla.org/adblockplus;1"].createInstance();
-  while (abp && !('getString' in abp))
-    abp = abp.wrappedJSObject;    // Unwrap component
+  abp = Components.classes["@mozilla.org/adblockplus;1"].createInstance().wrappedJSObject;
 
   if (abp.prefs.initialized) {
     var prefs = abp.prefs;
@@ -46,18 +44,26 @@ var wndData = null;
 var cacheSession = null;
 var noFlash = false;
 
+// Matchers for disabled filters
+var disabledBlacklistMatcher = new abp.Matcher();
+var disabledWhitelistMatcher = new abp.Matcher();
+
+function E(id) {
+  return document.getElementById(id);
+}
+
 function init() {
-  var list = document.getElementById("list");
+  var list = E("list");
   list.view = treeView;
 
   var selected = null;
   if (/sidebarDetached\.xul$/.test(parent.location.href)) {
     mainWin = parent.opener;
     mainWin.addEventListener("unload", mainUnload, false);
-    document.getElementById("detachButton").hidden = true;
-    document.getElementById("reattachButton").hidden = false;
+    E("detachButton").hidden = true;
+    E("reattachButton").hidden = false;
     if (!mainWin.document.getElementById("abp-sidebar"))
-      document.getElementById("reattachButton").setAttribute("disabled", "true");
+      E("reattachButton").setAttribute("disabled", "true");
     if (mainWin.document.getElementById("abp-key-sidebar")) {
       var sidebarKey = mainWin.document.getElementById("abp-key-sidebar").cloneNode(true);
       parent.document.getElementById("detached-keyset").appendChild(parent.document.importNode(sidebarKey, true));
@@ -69,16 +75,21 @@ function init() {
     // Install item listener
     DataContainer.addListener(handleItemChange);
 
+    // Initialize matchers for disabled filters
+    reloadDisabledFilters();
+    abp.filterStorage.addFilterObserver(reloadDisabledFilters);
+    abp.filterStorage.addSubscriptionObserver(reloadDisabledFilters);
+
     // Restore previous state
     var params = abp.getParams();
-    if (params && params.filter) {
-      document.getElementById("searchField").value = params.filter;
-      treeView.setFilter(params.filter);
+    if (params && params.search) {
+      E("searchField").value = params.search;
+      treeView.setFilter(params.search);
     }
-    if (params && params.focus && document.getElementById(params.focus))
-      document.getElementById(params.focus).focus();
+    if (params && params.focus && E(params.focus))
+      E(params.focus).focus();
     else
-      document.getElementById("searchField").focus();
+      E("searchField").focus();
 
     // Activate flasher
     list.addEventListener("select", onSelectionChange, false);
@@ -109,18 +120,40 @@ function cleanUp() {
 
   flasher.stop();
   DataContainer.removeListener(handleItemChange);
+  abp.filterStorage.removeFilterObserver(reloadDisabledFilters);
+  abp.filterStorage.removeSubscriptionObserver(reloadDisabledFilters);
 
   mainWin.abpGetBrowser().removeEventListener("select", handleTabChange, false);
   mainWin.removeEventListener("unload", mainUnload, false);
 }
 
+/**
+ * Updates matchers for disabled filters (global disabledBlacklistMatcher and
+ * disabledWhitelistMatcher variables), called on each filter change.
+ */
+function reloadDisabledFilters()
+{
+  disabledBlacklistMatcher.clear();
+  disabledWhitelistMatcher.clear();
+
+  for each (let subscription in abp.filterStorage.subscriptions)
+  {
+    if (subscription.disabled)
+      continue;
+
+    for each (let filter in subscription.filters)
+      if (filter instanceof abp.RegExpFilter && filter.disabled)
+        (filter instanceof abp.BlockingFilter ? disabledBlacklistMatcher : disabledWhitelistMatcher).add(filter);
+  }
+}
+
 // Called whenever list selection changes - triggers flasher
 function onSelectionChange() {
   var item = treeView.getSelectedItem();
   if (item)
-    document.getElementById("copy-command").removeAttribute("disabled");
+    E("copy-command").removeAttribute("disabled");
   else
-    document.getElementById("copy-command").setAttribute("disabled", "true");
+    E("copy-command").setAttribute("disabled", "true");
   if (item && wndData)
     wndData.lastSelection = item;
 
@@ -142,7 +175,7 @@ function handleItemChange(wnd, type, data, item) {
     type = "select";
 
   var i;
-  var filterSuggestions = document.getElementById("suggestionsList");
+  var filterSuggestions = E("suggestionsList");
   if (type == "clear") {
     // Current document has been unloaded, clear list
     wndData = null;
@@ -170,11 +203,14 @@ function handleTabChange() {
 }
 
 // Fills a box with text splitting it up into multiple lines if necessary
-function setMultilineContent(box, text) {
-  while (box.firstChild)
-    box.removeChild(box.firstChild);
+function setMultilineContent(box, text, noRemove)
+{
+  if (!noRemove)
+    while (box.firstChild)
+      box.removeChild(box.firstChild);
 
-  for (var i = 0; i < text.length; i += 80) {
+  for (var i = 0; i < text.length; i += 80)
+  {
     var description = document.createElement("description");
     description.setAttribute("value", text.substr(i, 80));
     box.appendChild(description);
@@ -192,33 +228,51 @@ function fillInTooltip(e) {
   if (!item)
     return false;
 
-  var filter = ("filter" in item ? item.filter : null);
+  let filter = ("filter" in item && item.filter && !item.filter.disabled ? item.filter : null);
+  let size = ("tooltip" in item ? null : getItemSize(item));
+  let subscriptions = (filter ? filter.subscriptions.filter(function(subscription) { return !subscription.disabled; }) : []);
 
-  document.getElementById("tooltipDummy").hidden = !("tooltip" in item);
-  document.getElementById("tooltipAddressRow").hidden = ("tooltip" in item);
-  document.getElementById("tooltipTypeRow").hidden = ("tooltip" in item);
-  document.getElementById("tooltipFilterRow").hidden = !filter;
+  E("tooltipDummy").hidden = !("tooltip" in item);
+  E("tooltipAddressRow").hidden = ("tooltip" in item);
+  E("tooltipTypeRow").hidden = ("tooltip" in item);
+  E("tooltipSizeRow").hidden = !size;
+  E("tooltipFilterRow").hidden = !filter;
+  E("tooltipFilterSourceRow").hidden = !subscriptions.length;
 
   if ("tooltip" in item)
-    document.getElementById("tooltipDummy").setAttribute("value", item.tooltip);
-  else {
-    document.getElementById("tooltipAddress").parentNode.hidden = (item.typeDescr == "ELEMHIDE");
-    setMultilineContent(document.getElementById("tooltipAddress"), item.location);
+    E("tooltipDummy").setAttribute("value", item.tooltip);
+  else
+  {
+    E("tooltipAddress").parentNode.hidden = (item.typeDescr == "ELEMHIDE");
+    setMultilineContent(E("tooltipAddress"), item.location);
   
     var type = item.localizedDescr;
-    if (filter && filter.type == "whitelist")
-      type += " " + document.getElementById("tooltipType").getAttribute("whitelisted");
+    if (filter && filter instanceof abp.WhitelistFilter)
+      type += " " + E("tooltipType").getAttribute("whitelisted");
     else if (filter && item.typeDescr != "ELEMHIDE")
-      type += " " + document.getElementById("tooltipType").getAttribute("filtered");
-    document.getElementById("tooltipType").setAttribute("value", type);
+      type += " " + E("tooltipType").getAttribute("filtered");
+    E("tooltipType").setAttribute("value", type);
+
+    if (size)
+      E("tooltipSize").setAttribute("value", size.join(" x "));
   }
 
   if (filter)
-    setMultilineContent(document.getElementById("tooltipFilter"), filter.text);
+  {
+    setMultilineContent(E("tooltipFilter"), filter.text);
+    if (subscriptions.length)
+    {
+      let sourceElement = E("tooltipFilterSource");
+      while (sourceElement.firstChild)
+        sourceElement.removeChild(sourceElement.firstChild);
+      for each (let subscription in subscriptions)
+        setMultilineContent(sourceElement, subscription.title, true);
+    }
+  }
 
   var showPreview = prefs.previewimages && !("tooltip" in item);
   showPreview = showPreview && (item.typeDescr == "IMAGE" || item.typeDescr == "BACKGROUND");
-  showPreview = showPreview && (!item.filter || item.filter.type == "whitelist");
+  showPreview = showPreview && (!item.filter || item.filter instanceof abp.WhitelistFilter);
   if (showPreview) {
     // Check whether image is in cache (stolen from ImgLikeOpera)
     if (!cacheSession) {
@@ -237,12 +291,12 @@ function fillInTooltip(e) {
   }
 
   if (showPreview) {
-    document.getElementById("tooltipPreviewBox").hidden = false;
-    document.getElementById("tooltipPreview").setAttribute("src", "");
-    document.getElementById("tooltipPreview").setAttribute("src", item.location);
+    E("tooltipPreviewBox").hidden = false;
+    E("tooltipPreview").setAttribute("src", "");
+    E("tooltipPreview").setAttribute("src", item.location);
   }
   else
-    document.getElementById("tooltipPreviewBox").hidden = true;
+    E("tooltipPreviewBox").hidden = true;
 
   return true;
 }
@@ -255,86 +309,168 @@ const visual = {
 
 // Fill in tooltip data before showing it
 function fillInContext(e) {
-  var item;
+  var item, allItems;
   if (treeView.data && !treeView.data.length)
+  {
     item = treeView.getDummyTooltip();
+    allItems = [item];
+  }
   else
+  {
     item = treeView.getItemAt(e.clientX, e.clientY);
+    allItems = treeView.getAllSelectedItems();
+  }
 
   if (!item || ("tooltip" in item && !("filter" in item)))
     return false;
 
-  document.getElementById("contextBlock").hidden = ("filter" in item && item.filter != null);
-  document.getElementById("contextEditFilter").hidden = !("filter" in item && item.filter != null);
-  document.getElementById("contextWhitelist").setAttribute("disabled", !!("tooltip" in item || item.typeDescr == "ELEMHIDE" || (item.filter && item.filter.type == "whitelist")));
-  document.getElementById("contextOpen").setAttribute("disabled", "tooltip" in item || item.typeDescr == "ELEMHIDE");
-  document.getElementById("contextFlash").setAttribute("disabled", !!("tooltip" in item || !(item.typeDescr in visual) || (item.filter && item.filter.type != "whitelist")));
+  E("contextDisableFilter").hidden = true;
+  E("contextEnableFilter").hidden = true;
+  if ("filter" in item && item.filter)
+  {
+    let filter = item.filter;
+    let menuItem = E(item.filter.disabled ? "contextEnableFilter" : "contextDisableFilter");
+    menuItem.filter = filter;
+    menuItem.setAttribute("label", menuItem.getAttribute("labeltempl").replace(/--/, filter.text));
+    menuItem.hidden = false;
+  }
+
+  E("contextWhitelist").hidden = ("tooltip" in item || !item.filter || item.filter.disabled || item.filter instanceof abp.WhitelistFilter || item.typeDescr == "ELEMHIDE");
+  E("contextBlock").hidden = !E("contextWhitelist").hidden;
+  E("contextBlock").setAttribute("disabled", "filter" in item && item.filter && !item.filter.disabled);
+  E("contextEditFilter").setAttribute("disabled", !("filter" in item && item.filter));
+  E("contextOpen").setAttribute("disabled", "tooltip" in item || item.typeDescr == "ELEMHIDE");
+  E("contextFlash").setAttribute("disabled", "tooltip" in item || !(item.typeDescr in visual) || (item.filter && !item.filter.disabled && !(item.filter instanceof abp.WhitelistFilter)));
+  E("contextCopyFilter").setAttribute("disabled", !allItems.some(function(item) {return "filter" in item && item.filter}));
 
   return true;
 }
 
-// Handles middle-click on an item
-function openInTab(e) {
-  var item = (typeof e == "undefined" ? treeView.getSelectedItem() : treeView.getItemAt(e.clientX, e.clientY));
-  if (!item || item.typeDescr == "ELEMHIDE")
-    return;
-
-  if ('delayedOpenTab' in mainWin)
-    mainWin.delayedOpenTab(item.location);
-  else if ('getBrowser' in mainWin)
-    mainWin.getBrowser().addTab(item.location);
-  else {
-    var uri = Components.classes["@mozilla.org/network/io-service;1"]
-                        .getService(Components.interfaces.nsIIOService)
-                        .newURI(item.location, null, null);
-
-    var protocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
-                                .getService(Components.interfaces.nsIExternalProtocolService);
-    protocolSvc.loadUrl(uri);
+/**
+ * Processed mouse clicks on the item list.
+ * @param {Event} event
+ */
+function handleClick(event)
+{
+  let item = treeView.getItemAt(event.clientX, event.clientY);
+  if (event.button == 0 && treeView.getColumnAt(event.clientX, event.clientY) == "state")
+  {
+    if (item.filter)
+      enableFilter(item.filter, item.filter.disabled);
+    event.preventDefault();
+  }
+  else if (event.button == 1)
+  {
+    openInTab(item);
+    event.preventDefault();
   }
 }
 
-// Starts up the main Adblock window
+/**
+ * Processes double-clicks on the item list.
+ * @param {Event} event
+ */
+function handleDblClick(event)
+{
+  if (event.button != 0 || treeView.getColumnAt(event.clientX, event.clientY) == "state")
+    return;
+
+  doBlock();
+}
+
+/**
+ * Opens the item in a new tab.
+ */
+function openInTab(item)
+{
+  if (!item)
+    item = treeView.getSelectedItem();
+  if (!item || item.typeDescr == "ELEMHIDE")
+    return;
+
+  abp.loadInBrowser(item.location, mainWin);
+}
+
 function doBlock() {
   if (!abp)
     return;
 
   var item = treeView.getSelectedItem();
-  if (treeView.data && !treeView.data.length) {
-    item = treeView.getDummyTooltip();
-    item.location = undefined;
-    if (!("filter" in item))
-      item.filter = null;
-  }
+  if (!item || item.typeDescr == "ELEMHIDE")
+    return;
 
-  var location = (item ? item.location : undefined);
-  var filter = (item && item.filter ? item.filter : undefined);
+  var filter = null;
+  if ("filter" in item)
+    filter = item.filter;
 
-  abp.openSettingsDialog(window.content, location, filter);
+  if (filter && filter instanceof abp.WhitelistFilter)
+    return;
+
+  openDialog("chrome://adblockplus/content/composer.xul", "_blank", "chrome,centerscreen,resizable,dialog=no,dependent", window.content, item);
 }
 
-function doWhitelist() {
+function editFilter() {
   if (!abp)
     return;
 
   var item = treeView.getSelectedItem();
-  if (!item)
+  if (treeView.data && !treeView.data.length)
+    item = treeView.getDummyTooltip();
+
+  if (!("filter" in item) || !item.filter)
     return;
 
-  abp.openSettingsDialog(window.content, "@@" + item.location);
+  if (!("location") in item)
+    item.location = undefined
+
+  abp.openSettingsDialog(item.location, item.filter);
+}
+
+function enableFilter(filter, enable) {
+  if (!abp)
+    return;
+
+  filter.disabled = !enable;
+  abp.filterStorage.triggerFilterObservers(enable ? "enable" : "disable", [filter]);
+  abp.filterStorage.saveToDisk();
+
+  treeView.boxObject.invalidate();
 }
 
 function copyToClipboard() {
   if (!abp)
     return;
 
-  var item = treeView.getSelectedItem();
-  if (!item)
+  var items = treeView.getAllSelectedItems();
+  if (!items.length)
     return;
 
   var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                                   .getService(Components.interfaces.nsIClipboardHelper);
-  clipboardHelper.copyString(item.location);
+  clipboardHelper.copyString(items.map(function(item) {return item.location}).join("\n"));
+}
+
+function copyFilter() {
+  if (!abp)
+    return;
+
+  var items = treeView.getAllSelectedItems().filter(function(item) {return item.filter});
+  if (treeView.data && !treeView.data.length)
+    items = [treeView.getDummyTooltip()];
+
+  if (!items.length)
+    return;
+
+  var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
+                                  .getService(Components.interfaces.nsIClipboardHelper);
+  clipboardHelper.copyString(items.map(function(item) {return item.filter.text}).join("\n"));
+}
+
+function selectAll() {
+  if (!abp)
+    return;
+
+  treeView.selectAll();
 }
 
 // Saves sidebar's state before detaching/reattaching
@@ -387,6 +523,22 @@ function reattach() {
   parent.close();
 }
 
+// Returns items size in the document if available
+function getItemSize(item)
+{
+  if (item.filter && !item.filter.disabled && item.filter instanceof abp.BlockingFilter)
+    return null;
+
+  for each (let node in item.nodes)
+  {
+    if (node instanceof HTMLImageElement && (node.naturalWidth || node.naturalHeight))
+      return [node.naturalWidth, node.naturalHeight];
+    else if (node instanceof HTMLElement && (node.offsetWidth || node.offsetHeight))
+      return [node.offsetWidth, node.offsetHeight];
+  }
+  return null;
+}
+
 // Sort functions for the item list
 function sortByAddress(item1, item2) {
   if (item1.location < item2.location)
@@ -424,11 +576,20 @@ function compareFilter(item1, item2) {
 }
 
 function compareState(item1, item2) {
-  var state1 = (!item1.filter ? 0 : (item1.filter.type == "whitelist" ? 1 : 2));
-  var state2 = (!item2.filter ? 0 : (item2.filter.type == "whitelist" ? 1 : 2));
+  var state1 = (!item1.filter ? 0 : (item1.filter.disabled ? 1 : (item1.filter instanceof abp.WhitelistFilter ? 2 : 3)));
+  var state2 = (!item2.filter ? 0 : (item2.filter.disabled ? 1 : (item2.filter instanceof abp.WhitelistFilter ? 2 : 3)));
   return state1 - state2;
 }
 
+function compareSize(item1, item2) {
+  var size1 = getItemSize(item1);
+  size1 = size1 ? size1[0] * size1[1] : 0;
+
+  var size2 = getItemSize(item2);
+  size2 = size2 ? size2[0] * size2[1] : 0;
+  return size1 - size2;
+}
+
 function createSortWithFallback(cmpFunc, fallbackFunc, desc) {
   var factor = (desc ? -1 : 1);
 
@@ -467,24 +628,23 @@ var treeView = {
     if (!boxObject)
       return;
 
-    var i;
-
     this.boxObject = boxObject;
     this.itemsDummy = boxObject.treeBody.getAttribute("noitemslabel");
     this.whitelistDummy = boxObject.treeBody.getAttribute("whitelistedlabel");
     this.loadDummy = boxObject.treeBody.getAttribute("notloadedlabel");
 
-    var stringAtoms = ["col-address", "col-type", "col-filter", "col-state", "state-regular", "state-filtered", "state-whitelisted"];
-    var boolAtoms = ["selected", "dummy"];
+    var stringAtoms = ["col-address", "col-type", "col-filter", "col-state", "col-size", "state-regular", "state-filtered", "state-whitelisted", "state-hidden"];
+    var boolAtoms = ["selected", "dummy", "filter-disabled"];
     var atomService = Components.classes["@mozilla.org/atom-service;1"]
                                 .getService(Components.interfaces.nsIAtomService);
 
     this.atoms = {};
-    for (i = 0; i < stringAtoms.length; i++)
-      this.atoms[stringAtoms[i]] = atomService.getAtom(stringAtoms[i]);
-    for (i = 0; i < boolAtoms.length; i++) {
-      this.atoms[boolAtoms[i] + "-true"] = atomService.getAtom(boolAtoms[i] + "-true");
-      this.atoms[boolAtoms[i] + "-false"] = atomService.getAtom(boolAtoms[i] + "-false");
+    for each (let atom in stringAtoms)
+      this.atoms[atom] = atomService.getAtom(atom);
+    for each (let atom in boolAtoms)
+    {
+      this.atoms[atom + "-true"] = atomService.getAtom(atom + "-true");
+      this.atoms[atom + "-false"] = atomService.getAtom(atom + "-false");
     }
 
     if (abp) {
@@ -495,7 +655,7 @@ var treeView = {
     // Check current sort direction
     var cols = document.getElementsByTagName("treecol");
     var sortDir = null;
-    for (i = 0; i < cols.length; i++) {
+    for (let i = 0; i < cols.length; i++) {
       var col = cols[i];
       var dir = col.getAttribute("sortDirection");
       if (dir && dir != "natural") {
@@ -503,9 +663,25 @@ var treeView = {
         sortDir = dir;
       }
     }
+    if (!this.sortColumn)
+    {
+      let defaultSort = E("list").getAttribute("defaultSort");
+      if (/^(\w+)\s+(ascending|descending)$/.test(defaultSort))
+      {
+        this.sortColumn = E(RegExp.$1);
+        if (this.sortColumn)
+        {
+          sortDir = RegExp.$2;
+          this.sortColumn.setAttribute("sortDirection", sortDir);
+        }
+      }
+    }
 
     if (sortDir)
+    {
       this.sortProc = this.sortProcs[this.sortColumn.id + (sortDir == "descending" ? "Desc" : "")];
+      E("list").setAttribute("defaultSort", " ");
+    }
 
     // Make sure to update the dummy row every two seconds
     setInterval(function(view) {
@@ -525,7 +701,7 @@ var treeView = {
     col = col.id;
 
     // Only two columns have text
-    if (col != "type" && col != "address" && col != "filter")
+    if (col != "type" && col != "address" && col != "filter" && col != "size")
       return "";
 
     if (this.data && this.data.length) {
@@ -536,6 +712,11 @@ var treeView = {
         return this.data[row].localizedDescr;
       else if (col == "filter")
         return (this.data[row].filter ? this.data[row].filter.text : "");
+      else if (col == "size")
+      {
+        let size = getItemSize(this.data[row]);
+        return (size ? size.join(" x ") : "");
+      }
       else
         return this.data[row].location;
     }
@@ -573,9 +754,20 @@ var treeView = {
     if (this.data && this.data.length) {
       properties.AppendElement(this.atoms["dummy-false"]);
 
+      let filter = this.data[row].filter;
+      if (filter)
+        properties.AppendElement(this.atoms["filter-disabled-" + filter.disabled]);
+
       state = "state-regular";
-      if (this.data[row].filter)
-        state = (this.data[row].filter.type == "whitelist" ? "state-whitelisted" : "state-filtered");
+      if (filter && !filter.disabled)
+      {
+        if (filter instanceof abp.WhitelistFilter)
+          state = "state-whitelisted";
+        else if (filter instanceof abp.BlockingFilter)
+          state = "state-filtered";
+        else if (filter instanceof abp.ElemHideFilter)
+          state = "state-hidden";
+      }
     }
     else {
       properties.AppendElement(this.atoms["dummy-true"]);
@@ -596,7 +788,7 @@ var treeView = {
   cycleHeader: function(col) {
     col = col.id;
 
-    col = document.getElementById(col);
+    col = E(col);
     if (!col)
       return;
 
@@ -663,6 +855,7 @@ var treeView = {
   allData: [],
   sortColumn: null,
   sortProc: null,
+  resortTimeout: null,
   itemsDummy: null,
   whitelistDummy: null,
   itemsDummyTooltip: null,
@@ -677,13 +870,22 @@ var treeView = {
     filter: createSortWithFallback(compareFilter, sortByAddress, false),
     filterDesc: createSortWithFallback(compareFilter, sortByAddress, true),
     state: createSortWithFallback(compareState, sortByAddress, false),
-    stateDesc: createSortWithFallback(compareState, sortByAddress, true)
+    stateDesc: createSortWithFallback(compareState, sortByAddress, true),
+    size: createSortWithFallback(compareSize, sortByAddress, false),
+    sizeDesc: createSortWithFallback(compareSize, sortByAddress, true)
   },
 
   setData: function(data) {
     var oldRows = this.rowCount;
 
     this.allData = data;
+    for each (let item in this.allData)
+    {
+      if (!item.filter)
+        item.filter = disabledWhitelistMatcher.matchesAny(item.location, item.typeDescr, item.docDomain, item.thirdParty);
+      if (!item.filter)
+        item.filter = disabledBlacklistMatcher.matchesAny(item.location, item.typeDescr, item.docDomain, item.thirdParty);
+    }
     this.refilter();
 
     this.boxObject.rowCountChanged(0, -oldRows);
@@ -692,11 +894,31 @@ var treeView = {
 
   addItem: function(item) {
     this.allData.push(item);
-    if (this.filter && item.location.toLowerCase().indexOf(this.filter) < 0 && item.localizedDescr.toLowerCase().indexOf(this.filter) < 0)
+    if (!item.filter)
+      item.filter = disabledWhitelistMatcher.matchesAny(item.location, item.typeDescr, item.docDomain, item.thirdParty);
+    if (!item.filter)
+      item.filter = disabledBlacklistMatcher.matchesAny(item.location, item.typeDescr, item.docDomain, item.thirdParty);
+
+    if (!this.matchesFilter(item))
       return;
 
     var index = -1;
-    if (this.sortProc)
+    if (this.sortProc && this.sortColumn && this.sortColumn.id == "size")
+    {
+      // Sorting by size requires accessing content document, and that's
+      // dangerous from a content policy (and we are likely called directly
+      // from a content policy call). Size data will be inaccurate anyway,
+      // delay sorting until later.
+      if (this.resortTimeout)
+        clearTimeout(this.resortTimeout);
+      this.resortTimeout = setTimeout(function(me)
+      {
+        if (me.sortProc)
+          me.data.sort(me.sortProc);
+        me.boxObject.invalidate();
+      }, 500, this);
+    }
+    else if (this.sortProc)
       for (var i = 0; index < 0 && i < this.data.length; i++)
         if (this.sortProc(item, this.data[i]) < 0)
           index = i;
@@ -714,16 +936,34 @@ var treeView = {
       this.boxObject.rowCountChanged(index, 1);
   },
 
-  refilter: function() {
-    this.data = [];
-    for (var i = 0; i < this.allData.length; i++)
-      if (!this.filter || this.allData[i].location.toLowerCase().indexOf(this.filter) >= 0 || this.allData[i].localizedDescr.toLowerCase().indexOf(this.filter) >= 0)
-        this.data.push(this.allData[i]);
+  /**
+   * Updates the list after a filter or sorting change.
+   */
+  refilter: function()
+  {
+    if (this.resortTimeout)
+      clearTimeout(this.resortTimeout);
+
+    this.data = this.allData.filter(this.matchesFilter, this);
 
     if (this.sortProc)
       this.data.sort(this.sortProc);
   },
 
+  /**
+   * Tests whether an item matches current list filter.
+   * @return {Boolean} true if the item should be shown
+   */
+  matchesFilter: function(item)
+  {
+    if (!this.filter)
+      return true;
+
+    return (item.location.toLowerCase().indexOf(this.filter) >= 0 ||
+            (item.filter && item.filter.text.toLowerCase().indexOf(this.filter) >= 0) ||
+            item.localizedDescr.toLowerCase().indexOf(this.filter) >= 0);
+  },
+
   setFilter: function(filter) {
     var oldRows = this.rowCount;
 
@@ -736,6 +976,10 @@ var treeView = {
     this.boxObject.invalidate();
   },
 
+  selectAll: function() {
+    this.selection.selectAll();
+  },
+
   getSelectedItem: function() {
     if (!this.data || this.selection.currentIndex < 0 || this.selection.currentIndex >= this.data.length)
       return null;
@@ -743,7 +987,28 @@ var treeView = {
     return this.data[this.selection.currentIndex];
   },
 
-  getItemAt: function(x, y) {
+  getAllSelectedItems: function() {
+    let result = [];
+    if (!this.data)
+      return result;
+
+    let numRanges = this.selection.getRangeCount();
+    for (let i = 0; i < numRanges; i++)
+    {
+      let min = {};
+      let max = {};
+      let range = this.selection.getRangeAt(i, min, max);
+      for (let j = min.value; j <= max.value; j++)
+      {
+        if (j >= 0 && j < this.data.length)
+          result.push(this.data[j]);
+      }
+    }
+    return result;
+  },
+
+  getItemAt: function(x, y)
+  {
     if (!this.data)
       return null;
 
@@ -754,6 +1019,17 @@ var treeView = {
     return this.data[row];
   },
 
+  getColumnAt: function(x, y)
+  {
+    if (!this.data)
+      return null;
+
+    let col = {};
+    this.boxObject.getCellAt(x, y, {}, col, {});
+    if (col.value)
+      return col.value.id;
+  },
+
   getDummyTooltip: function() {
     if (!this.data || this.data.length)
       return null;
