@@ -29,13 +29,15 @@
 
 var ehhAardvark = {
   browser: null,
+  currentElem: null,
   selectedElem: null,
   commentElem : null,
   mouseX: -1,
   mouseY: -1,
   commandLabelTimeout: 0,
   borderElems: null,
-  labelElem: null
+  labelElem: null,
+  locked: false
 };
 
 ehhAardvark.start = function(browser) {
@@ -165,14 +167,15 @@ ehhAardvark.onMouseOver = function(event) {
 
   if (elem == null || aardvarkLabel)
   {
-    this.clearBox ();
+    this.currentElem = null;
+    if (!this.locked)
+        this.clearBox();
     return;
   }
 
-  if (elem == this.selectedElem)
-    return;
-  
-  this.showBoxAndLabel (elem, this.makeElementLabelString (elem));
+  this.currentElem = elem;
+  if (!this.locked && this.currentElem != this.selectedElem)
+    this.showBoxAndLabel(this.currentElem, this.makeElementLabelString(this.currentElem));
 }
 
 ehhAardvark.onKeyPress = function(event) {
@@ -437,6 +440,7 @@ ehhAardvark.setElementStyleDefault = function (elem, bgColor)
 // 0: name, 1: needs element
 ehhAardvark.commands = [
   "select",
+  "lock",
   "wider",
   "narrower",
   "quit",
@@ -525,6 +529,15 @@ ehhAardvark.select = function (elem)
   return true;
 }
 
+//------------------------------------------------------------
+ehhAardvark.lock = function (elem)
+{
+  this.locked = !this.locked;
+  if (!this.locked && this.currentElem != this.selectedElem)
+    this.showBoxAndLabel(this.currentElem, this.makeElementLabelString(this.currentElem));
+  return true;
+}
+
 //------------------------------------------------------------
 ehhAardvark.blinkElement = function (elem)
 {
@@ -562,10 +575,12 @@ ehhAardvark.viewSource = function (elem)
     return false;
 
   var sourceBox = document.getElementById("ehh-viewsource");
-  if (sourceBox.getAttribute("_moz-menuactive") == "true" && this.commentElem == elem) {
+  if ((sourceBox.getAttribute("_moz-menuactive") == "true" || sourceBox.state == "open") && this.commentElem == elem) {
+    ehhViewSourceHidden = true;
     sourceBox.hidePopup();
     return true;
   }
+  ehhViewSourceHidden = true;
   sourceBox.hidePopup();
 
   while (sourceBox.firstChild)
@@ -576,6 +591,7 @@ ehhAardvark.viewSource = function (elem)
   var x = this.mouseX;
   var y = this.mouseY;
   setTimeout(function() {
+    ehhViewSourceHidden = false;
     sourceBox.showPopup(document.documentElement, x, y, "tooltip", "topleft", "topleft");
   }, 500);
   return true;
@@ -676,12 +692,14 @@ ehhAardvark.getOuterHtmlFormatted = function (node, container)
 ehhAardvark.showMenu = function ()
 {
   var helpBox = document.getElementById("ehh-helpbox");
-  if (helpBox.getAttribute("_moz-menuactive") == "true") {
+  if (helpBox.getAttribute("_moz-menuactive") == "true" || helpBox.state == "open") {
+    ehhHelpHidden = true;
     helpBox.hidePopup();
     return true;
   }
 
   // Show help box
+  ehhHelpHidden = false;
   helpBox.showPopup(this.browser, -1, -1, "tooltip", "topleft", "topleft");
   return true;
 }
